@@ -35,6 +35,9 @@
 #include <auparse.h>
 #include <ctype.h>
 #include <sys/param.h>
+#include <limits.h>
+#include <syslog.h>
+
 
 /*
  * Defauls
@@ -43,7 +46,6 @@
 #define CFG_DEFAULT_INIT	"/sbin/init"
 #define CFG_DEFAULT_RAWLOG	NULL
 #define CFG_DEFAULT_LISTSPATH	NULL
-#define CFG_DEFAULT_EXECIGN	"/usr/sbin/readahead"
 #define CFG_DEFAULT_LISTSEP	"/etc/init.d/readahead_later"
 
 #define MAX_READLINKS 32
@@ -151,7 +153,7 @@ typedef struct
 	char		*initpath;
 	char		*listbgn;
 	char		*listsep;
-	char		*execign;
+	char		**execign;
 	char		*debuglog;
 	char		**excludes;
 	char		**argv;
@@ -190,7 +192,7 @@ static int list_write(RacStatus *rac, char **list, int size,
 static int blocks_dump_as_lists(RacStatus *rac, const char *filename_tpl);
 static char *btrim(char *str);
 static char **string_to_array(char *data);
-static int exclude_file(RacStatus *rac, const char *filename);
+static int exclude_file(char **list, const char *filename);
 
 int
 main(int argc, char **argv)
@@ -221,7 +223,6 @@ main(int argc, char **argv)
 	rac->initpath = CFG_DEFAULT_INIT;
 	rac->rawlog = CFG_DEFAULT_RAWLOG;
 	rac->listspath = CFG_DEFAULT_LISTSPATH;
-	rac->execign = CFG_DEFAULT_EXECIGN;
 	rac->listsep = CFG_DEFAULT_LISTSEP;
 	rac->argv = argv;
 	rac->argc = argc;
@@ -285,13 +286,17 @@ main(int argc, char **argv)
 	atexit(clean_audit);
 
 	/* Open way to audit */
-	if ((audit_fd = audit_open()) < 0)
+	if ((audit_fd = audit_open()) < 0) {
+		daemon_child_status(CHILD_FAILURE);
 		die("cannot open netlink audit socket");
+	}
 	rac->fd = audit_fd;
 
 	/* Tell the kernel we are alive */
-	if (audit_set_pid(rac->fd, getpid(), WAIT_YES) < 0)
+	if (audit_set_pid(rac->fd, getpid(), WAIT_YES) < 0) {
+		daemon_child_status(CHILD_FAILURE);
 		die("cannot set PID");
+	}
 
 	/* Tell parent that everything os OK */
 	if (!is_foreground(rac)) {
@@ -427,7 +432,7 @@ cfg_read(RacStatus *rac)
 		else if ((p = cfg_get_item(buff, "RAC_LISTSPATH")))
 			rac->listspath = strdup(p);
 		else if ((p = cfg_get_item(buff, "RAC_EXECIGN")))
-			rac->execign = strdup(p);
+			rac->execign = string_to_array(p);
 		else if ((p = cfg_get_item(buff, "RAC_LISTBEGIN")))
 			rac->listbgn = strdup(p);
 		else if ((p = cfg_get_item(buff, "RAC_LISTSEP")))
@@ -1023,9 +1028,6 @@ parse_events(RacStatus *rac, char **ary, int *_nfiles, int *sep)
 	if (!all)
 		return NULL;
 
-	if (rac->execign)
-		ignlen = strlen(rac->execign);
-
 	debug("parsing buffers: start (events: %d)", rac->nevents);
 
 	do {
@@ -1043,9 +1045,9 @@ parse_events(RacStatus *rac, char **ary, int *_nfiles, int *sep)
 			int type = auparse_get_type(au);
 			const char *p;
 
-			if (ignlen && type == AUDIT_SYSCALL &&
+			if (type == AUDIT_SYSCALL &&
 					(p = auparse_find_field(au, "exe"))) {
-				if (strncmp(rac->execign, p+1, ignlen) == 0)
+				if (exclude_file(rac->execign, p+1))
 					wanted = 0;
 			}
 			else if (type == AUDIT_CWD &&
@@ -1103,7 +1105,7 @@ parse_events(RacStatus *rac, char **ary, int *_nfiles, int *sep)
 			if (nfiles && strcmp(all[nfiles-1], buff) == 0)
 				continue;
 
-			if (exclude_file(rac, buff))
+			if (exclude_file(rac->excludes, buff))
 				continue;
 
 			if (stat(buff, &st) == -1 || S_ISREG(st.st_mode) == 0)
@@ -1249,11 +1251,11 @@ myrealpath(const char *path, char *resolved_path, int maxreslth, const char *cwd
  * returns 1 if file should be excluded
  */
 static int
-exclude_file(RacStatus *rac, const char *filename)
+exclude_file(char **list, const char *filename)
 {
-	char **p;
+	char **p = list;
 
-	for (p = rac->excludes; p && *p; p++) {
+	for (; p && *p; p++) {
 		const char *f = filename;
 		char *e = *p;
 		while(*f++ == *e++);
